(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react-dom'), require('react')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react-dom', 'react'], factory) :
  (global = global || self, factory(global.ReactQuery = {}, global.ReactDOM, global.React));
}(this, (function (exports, ReactDOM, React) { 'use strict';

  ReactDOM = ReactDOM && Object.prototype.hasOwnProperty.call(ReactDOM, 'default') ? ReactDOM['default'] : ReactDOM;
  React = React && Object.prototype.hasOwnProperty.call(React, 'default') ? React['default'] : React;

  function _extends() {
    _extends = Object.assign || function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];

        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }

      return target;
    };

    return _extends.apply(this, arguments);
  }

  // TYPES
  var CancelledError = function CancelledError(silent) {
    this.silent = silent;
  }; // UTILS

  var isServer = typeof window === 'undefined';
  function noop() {
    return undefined;
  }
  function functionalUpdate(updater, input) {
    return typeof updater === 'function' ? updater(input) : updater;
  }

  function stableStringifyReplacer(_key, value) {
    if (typeof value === 'function') {
      throw new Error();
    }

    if (isPlainObject(value)) {
      return Object.keys(value).sort().reduce(function (result, key) {
        result[key] = value[key];
        return result;
      }, {});
    }

    return value;
  }

  function stableStringify(value) {
    return JSON.stringify(value, stableStringifyReplacer);
  }

  function defaultRetryDelay(attempt) {
    return Math.min(1000 * Math.pow(2, attempt), 30000);
  }
  function defaultQueryKeySerializerFn(queryKey) {
    try {
      return stableStringify(queryKey);
    } catch (_unused) {
      throw new Error('Failed to serialize query key');
    }
  }
  function hashQueryKey(queryKey, options) {
    return (options == null ? void 0 : options.queryKeySerializerFn) ? options.queryKeySerializerFn(queryKey) : defaultQueryKeySerializerFn(queryKey);
  }
  function deepIncludes(a, b) {
    if (a === b) {
      return true;
    }

    if (typeof a !== typeof b) {
      return false;
    }

    if (typeof a === 'object') {
      return !Object.keys(b).some(function (key) {
        return !deepIncludes(a[key], b[key]);
      });
    }

    return false;
  }
  function isValidTimeout(value) {
    return typeof value === 'number' && value >= 0 && value !== Infinity;
  }
  function isDocumentVisible() {
    // document global can be unavailable in react native
    if (typeof document === 'undefined') {
      return true;
    }

    return [undefined, 'visible', 'prerender'].includes(document.visibilityState);
  }
  function isOnline() {
    return navigator.onLine === undefined || navigator.onLine;
  }
  function ensureArray(value) {
    return Array.isArray(value) ? value : [value];
  }
  function uniq(array) {
    return array.filter(function (value, i) {
      return array.indexOf(value) === i;
    });
  }
  function difference(array1, array2) {
    return array1.filter(function (x) {
      return array2.indexOf(x) === -1;
    });
  }
  function replaceAt(array, index, value) {
    var copy = array.slice(0);
    copy[index] = value;
    return copy;
  }
  function timeUntilStale(updatedAt, staleTime) {
    return Math.max(updatedAt + (staleTime || 0) - Date.now(), 0);
  }
  function parseQueryArgs(arg1, arg2, arg3) {
    if (!isQueryKey(arg1)) {
      return arg1;
    }

    if (typeof arg2 === 'function') {
      return _extends({}, arg3, {
        queryKey: arg1,
        queryFn: arg2
      });
    }

    return _extends({}, arg2, {
      queryKey: arg1
    });
  }
  function parseFilterArgs(arg1, arg2, arg3) {
    return isQueryKey(arg1) ? [_extends({}, arg2, {
      queryKey: arg1
    }), arg3] : [arg1 || {}, arg2];
  }
  function matchQuery(filters, query) {
    var active = filters.active,
        exact = filters.exact,
        fetching = filters.fetching,
        fresh = filters.fresh,
        inactive = filters.inactive,
        predicate = filters.predicate,
        queryKey = filters.queryKey,
        stale = filters.stale;

    if (queryKey && (exact ? query.queryHash !== hashQueryKey(queryKey, query.options) : !deepIncludes(query.queryKey, queryKey))) {
      return false;
    }

    var isActive;

    if (inactive === false || active && !inactive) {
      isActive = true;
    } else if (active === false || inactive && !active) {
      isActive = false;
    }

    if (typeof isActive === 'boolean' && query.isActive() !== isActive) {
      return false;
    }

    var isStale;

    if (fresh === false || stale && !fresh) {
      isStale = true;
    } else if (stale === false || fresh && !stale) {
      isStale = false;
    }

    if (typeof isStale === 'boolean' && query.isStale() !== isStale) {
      return false;
    }

    if (typeof fetching === 'boolean' && query.isFetching() !== fetching) {
      return false;
    }

    if (predicate && !predicate(query)) {
      return false;
    }

    return true;
  }
  /**
   * This function returns `a` if `b` is deeply equal.
   * If not, it will replace any deeply equal children of `b` with those of `a`.
   * This can be used for structural sharing between JSON values for example.
   */

  function replaceEqualDeep(a, b) {
    if (a === b) {
      return a;
    }

    var array = Array.isArray(a) && Array.isArray(b);

    if (array || isPlainObject(a) && isPlainObject(b)) {
      var aSize = array ? a.length : Object.keys(a).length;
      var bItems = array ? b : Object.keys(b);
      var bSize = bItems.length;
      var copy = array ? [] : {};
      var equalItems = 0;

      for (var i = 0; i < bSize; i++) {
        var key = array ? i : bItems[i];
        copy[key] = replaceEqualDeep(a[key], b[key]);

        if (copy[key] === a[key]) {
          equalItems++;
        }
      }

      return aSize === bSize && equalItems === aSize ? a : copy;
    }

    return b;
  }
  /**
   * Shallow compare objects. Only works with objects that always have the same properties.
   */

  function shallowEqualObjects(a, b) {
    for (var key in a) {
      if (a[key] !== b[key]) {
        return false;
      }
    }

    return true;
  } // Copied from: https://github.com/jonschlinkert/is-plain-object

  function isPlainObject(o) {
    if (!hasObjectPrototype(o)) {
      return false;
    } // If has modified constructor


    var ctor = o.constructor;

    if (typeof ctor === 'undefined') {
      return true;
    } // If has modified prototype


    var prot = ctor.prototype;

    if (!hasObjectPrototype(prot)) {
      return false;
    } // If constructor does not have an Object-specific method


    if (!prot.hasOwnProperty('isPrototypeOf')) {
      return false;
    } // Most likely a plain Object


    return true;
  }

  function hasObjectPrototype(o) {
    return Object.prototype.toString.call(o) === '[object Object]';
  }

  function isQueryKey(value) {
    return typeof value === 'string' || Array.isArray(value);
  }
  function isCancelable(value) {
    return typeof (value == null ? void 0 : value.cancel) === 'function';
  }
  function isError(value) {
    return value instanceof Error;
  }
  function isCancelledError(value) {
    return value instanceof CancelledError;
  }
  function sleep(timeout) {
    return new Promise(function (resolve) {
      setTimeout(resolve, timeout);
    });
  }
  function getStatusProps(status) {
    return {
      status: status,
      isLoading: status === 'loading',
      isSuccess: status === 'success',
      isError: status === 'error',
      isIdle: status === 'idle'
    };
  }
  function createSetHandler(fn) {
    var removePreviousHandler;
    return function (callback) {
      // Unsub the old handler
      if (removePreviousHandler) {
        removePreviousHandler();
      } // Sub the new handler


      removePreviousHandler = callback(fn);
    };
  }
  /**
   * Schedules a microtask.
   * This can be useful to schedule state updates after rendering.
   */

  function scheduleMicrotask(callback) {
    Promise.resolve().then(callback).catch(function (error) {
      return setTimeout(function () {
        throw error;
      });
    });
  }

  // CLASS
  var NotifyManager = /*#__PURE__*/function () {
    function NotifyManager() {
      this.queue = [];
      this.transactions = 0;
    }

    var _proto = NotifyManager.prototype;

    _proto.batch = function batch(callback) {
      this.transactions++;
      var result = callback();
      this.transactions--;

      if (!this.transactions) {
        this.flush();
      }

      return result;
    };

    _proto.schedule = function schedule(notify) {
      if (this.transactions) {
        this.queue.push(notify);
      } else {
        scheduleMicrotask(function () {
          notify();
        });
      }
    };

    _proto.flush = function flush() {
      var queue = this.queue;
      this.queue = [];

      if (queue.length) {
        scheduleMicrotask(function () {
          var batchedUpdates = getBatchedUpdates();
          batchedUpdates(function () {
            queue.forEach(function (notify) {
              notify();
            });
          });
        });
      }
    };

    return NotifyManager;
  }(); // GETTERS AND SETTERS
  // Default to a dummy "batch" implementation that just runs the callback

  var batchedUpdates = function batchedUpdates(callback) {
    callback();
  }; // Allow injecting another batching function later


  function setBatchedUpdates(fn) {
    batchedUpdates = fn;
  } // Supply a getter just to skip dealing with ESM bindings

  function getBatchedUpdates() {
    return batchedUpdates;
  } // SINGLETON

  var notifyManager = new NotifyManager();

  // FUNCTIONS
  var consoleObject = console || {
    error: noop,
    warn: noop,
    log: noop
  };
  function getConsole() {
    return consoleObject;
  }
  function setConsole(c) {
    consoleObject = c;
  }

  // CLASS
  function _empty() {}

  function _awaitIgnored(value, direct) {
    if (!direct) {
      return value && value.then ? value.then(_empty) : Promise.resolve();
    }
  }

  function _invoke(body, then) {
    var result = body();

    if (result && result.then) {
      return result.then(then);
    }

    return then(result);
  }

  function _await(value, then, direct) {
    if (direct) {
      return then ? then(value) : value;
    }

    if (!value || !value.then) {
      value = Promise.resolve(value);
    }

    return then ? value.then(then) : value;
  }

  function _catch(body, recover) {
    try {
      var result = body();
    } catch (e) {
      return recover(e);
    }

    if (result && result.then) {
      return result.then(void 0, recover);
    }

    return result;
  }

  function _async(f) {
    return function () {
      for (var args = [], i = 0; i < arguments.length; i++) {
        args[i] = arguments[i];
      }

      try {
        return Promise.resolve(f.apply(this, args));
      } catch (e) {
        return Promise.reject(e);
      }
    };
  }

  var Query = /*#__PURE__*/function () {
    function Query(config) {
      var _config$queryHash;

      this.setOptions(config.options);
      this.observers = [];
      this.cache = config.cache;
      this.queryKey = config.queryKey;
      this.queryHash = (_config$queryHash = config.queryHash) != null ? _config$queryHash : hashQueryKey(config.queryKey, this.options);
      this.state = getDefaultState(this.options);
      this.scheduleGc();
    }

    var _proto = Query.prototype;

    _proto.setOptions = function setOptions(options) {
      var _this$options$cacheTi;

      this.options = _extends({}, this.defaultOptions, options); // Default to 5 minutes if not cache time is set

      this.cacheTime = Math.max(this.cacheTime || 0, (_this$options$cacheTi = this.options.cacheTime) != null ? _this$options$cacheTi : 5 * 60 * 1000);
    };

    _proto.setDefaultOptions = function setDefaultOptions(options) {
      this.defaultOptions = options;
    };

    _proto.dispatch = function dispatch(action) {
      var _this = this;

      this.state = queryReducer(this.state, action);
      notifyManager.batch(function () {
        _this.observers.forEach(function (observer) {
          observer.onQueryUpdate(action);
        });

        _this.cache.notify(_this);
      });
    };

    _proto.scheduleGc = function scheduleGc() {
      var _this2 = this;

      this.clearGcTimeout();

      if (isServer || this.observers.length > 0 || !isValidTimeout(this.cacheTime)) {
        return;
      }

      this.gcTimeout = setTimeout(function () {
        _this2.remove();
      }, this.cacheTime);
    };

    _proto.cancel = function cancel(silent) {
      var promise = this.promise;

      if (promise && this.cancelFetch) {
        this.cancelFetch(silent);
        return promise.then(noop).catch(noop);
      }

      return Promise.resolve(undefined);
    };

    _proto.clearTimersObservers = function clearTimersObservers() {
      this.observers.forEach(function (observer) {
        observer.clearTimers();
      });
    };

    _proto.clearGcTimeout = function clearGcTimeout() {
      clearTimeout(this.gcTimeout);
      this.gcTimeout = undefined;
    };

    _proto.setData = function setData(updater, options) {
      var _this$options$isDataE, _this$options;

      var prevData = this.state.data; // Get the new data

      var data = functionalUpdate(updater, prevData); // Structurally share data between prev and new data if needed

      if (this.options.structuralSharing !== false) {
        data = replaceEqualDeep(prevData, data);
      } // Use prev data if an isDataEqual function is defined and returns `true`


      if ((_this$options$isDataE = (_this$options = this.options).isDataEqual) == null ? void 0 : _this$options$isDataE.call(_this$options, prevData, data)) {
        data = prevData;
      } // Try to determine if more data can be fetched


      var canFetchMore = hasMorePages(this.options, data); // Set data and mark it as cached

      this.dispatch({
        type: 'success',
        data: data,
        canFetchMore: canFetchMore,
        updatedAt: options == null ? void 0 : options.updatedAt
      });
      return data;
    };

    _proto.remove = function remove() {
      this.cache.remove(this);
    };

    _proto.destroy = function destroy() {
      this.clearGcTimeout();
      this.clearTimersObservers();
      this.cancel();
    };

    _proto.isActive = function isActive() {
      return this.observers.some(function (observer) {
        return observer.options.enabled !== false;
      });
    };

    _proto.isFetching = function isFetching() {
      return Boolean(this.promise);
    };

    _proto.isStale = function isStale() {
      return this.state.isInvalidated || !this.state.updatedAt || this.observers.some(function (observer) {
        return observer.getCurrentResult().isStale;
      });
    };

    _proto.isStaleByTime = function isStaleByTime(staleTime) {
      if (staleTime === void 0) {
        staleTime = 0;
      }

      return this.state.isInvalidated || !this.state.updatedAt || !timeUntilStale(this.state.updatedAt, staleTime);
    };

    _proto.onFocus = function onFocus() {
      this.onExternalEvent('focus');
    };

    _proto.onOnline = function onOnline() {
      this.onExternalEvent('online');
    };

    _proto.onExternalEvent = function onExternalEvent(type) {
      var _this$continueFetch;

      // Execute the first observer that wants to fetch on this event
      var fetchObserver = this.observers.find(function (observer) {
        var _observer$options = observer.options,
            enabled = _observer$options.enabled,
            refetchOnWindowFocus = _observer$options.refetchOnWindowFocus,
            refetchOnReconnect = _observer$options.refetchOnReconnect;

        var _observer$getCurrentR = observer.getCurrentResult(),
            isStale = _observer$getCurrentR.isStale;

        return enabled !== false && (type === 'focus' && (refetchOnWindowFocus === 'always' || refetchOnWindowFocus !== false && isStale) || type === 'online' && (refetchOnReconnect === 'always' || refetchOnReconnect !== false && isStale));
      });

      if (fetchObserver) {
        fetchObserver.refetch();
      } // Continue any paused fetch


      (_this$continueFetch = this.continueFetch) == null ? void 0 : _this$continueFetch.call(this);
    };

    _proto.subscribeObserver = function subscribeObserver(observer) {
      if (this.observers.indexOf(observer) !== -1) {
        return;
      }

      this.observers.push(observer); // Stop the query from being garbage collected

      this.clearGcTimeout();
      this.cache.notify(this);
    };

    _proto.unsubscribeObserver = function unsubscribeObserver(observer) {
      this.observers = this.observers.filter(function (x) {
        return x !== observer;
      });

      if (!this.observers.length) {
        // If the transport layer does not support cancellation
        // we'll let the query continue so the result can be cached
        if (this.isTransportCancelable) {
          this.cancel();
        }

        this.scheduleGc();
      }

      this.cache.notify(this);
    };

    _proto.invalidate = function invalidate() {
      if (!this.state.isInvalidated) {
        this.dispatch({
          type: 'invalidate'
        });
      }
    };

    _proto.fetch = function fetch(options, fetchOptions) {
      try {
        var _exit2 = false;

        var _this4 = this;

        return _invoke(function () {
          if (_this4.promise) {
            return function () {
              if ((fetchOptions == null ? void 0 : fetchOptions.fetchMore) && _this4.state.data) {
                // Silently cancel current fetch if the user wants to fetch more
                return _awaitIgnored(_this4.cancel(true));
              } else {
                // Return current promise if we are already fetching
                _exit2 = true;
                return _this4.promise;
              }
            }();
          }
        }, function (_result2) {
          if (_exit2) return _result2;

          // Update config if passed, otherwise the config from the last execution is used
          if (options) {
            _this4.setOptions(options);
          } // Use the options from the first observer with a query function if no function is found.
          // This can happen when the query is hydrated or created with setQueryData.


          if (!_this4.options.queryFn) {
            var observer = _this4.observers.find(function (x) {
              return x.options.queryFn;
            });

            if (observer) {
              _this4.setOptions(observer.options);
            }
          }

          options = _this4.options; // Get the query function params

          var params = ensureArray(_this4.queryKey);
          var filter = options.queryFnParamsFilter;
          params = filter ? filter(params) : params;
          _this4.promise = _async(function () {
            return _catch(function () {
              var data;
              return _invoke(function () {
                if (options.infinite) {
                  return _await(_this4.startInfiniteFetch(options, params, fetchOptions), function (_this4$startInfiniteF) {
                    data = _this4$startInfiniteF;
                  });
                } else {
                  return _await(_this4.startFetch(options, params), function (_this4$startFetch) {
                    data = _this4$startFetch;
                  });
                }
              }, function () {
                // Set success state
                _this4.setData(data); // Cleanup


                delete _this4.promise; // Return data

                return data;
              });
            }, function (error) {
              // Set error state if needed
              if (!(isCancelledError(error) && error.silent)) {
                _this4.dispatch({
                  type: 'error',
                  error: error
                });
              } // Log error


              if (!isCancelledError(error)) {
                getConsole().error(error);
              } // Cleanup


              delete _this4.promise; // Propagate error

              throw error;
            });
          })();
          return _this4.promise;
        });
      } catch (e) {
        return Promise.reject(e);
      }
    };

    _proto.startFetch = function startFetch(options, params) {
      // Create function to fetch the data
      var queryFn = options.queryFn || defaultQueryFn;

      var fetchData = function fetchData() {
        return queryFn.apply(void 0, params);
      }; // Set to fetching state if not already in it


      if (!this.state.isFetching) {
        this.dispatch({
          type: 'fetch'
        });
      } // Try to fetch the data


      return this.tryFetchData(options, fetchData);
    };

    _proto.startInfiniteFetch = function startInfiniteFetch(options, params, fetchOptions) {
      var fetchMore = fetchOptions == null ? void 0 : fetchOptions.fetchMore;

      var _ref = fetchMore || {},
          previous = _ref.previous,
          fetchMoreVariable = _ref.fetchMoreVariable;

      var isFetchingMore = fetchMore ? previous ? 'previous' : 'next' : false;
      var prevPages = this.state.data || []; // Create function to fetch a page

      var fetchPage = _async(function (pages, prepend, cursor) {
        var lastPage = getLastPage(pages, prepend);

        if (typeof cursor === 'undefined' && typeof lastPage !== 'undefined' && options.getFetchMore) {
          cursor = options.getFetchMore(lastPage, pages);
        }

        var queryFn = options.queryFn || defaultQueryFn;
        return _await(queryFn.apply(void 0, params.concat([cursor])), function (page) {
          return prepend ? [page].concat(pages) : [].concat(pages, [page]);
        });
      }); // Create function to fetch the data


      var fetchData = function fetchData() {
        if (isFetchingMore) {
          return fetchPage(prevPages, previous, fetchMoreVariable);
        } else if (!prevPages.length) {
          return fetchPage([]);
        } else {
          var promise = fetchPage([]);

          for (var i = 1; i < prevPages.length; i++) {
            promise = promise.then(fetchPage);
          }

          return promise;
        }
      }; // Set to fetching state if not already in it


      if (!this.state.isFetching || this.state.isFetchingMore !== isFetchingMore) {
        this.dispatch({
          type: 'fetch',
          isFetchingMore: isFetchingMore
        });
      } // Try to get the data


      return this.tryFetchData(options, fetchData);
    };

    _proto.tryFetchData = function tryFetchData(options, fn) {
      var _this5 = this;

      return new Promise(function (outerResolve, outerReject) {
        var resolved = false;
        var continueLoop;
        var cancelTransport;

        var done = function done() {
          resolved = true;
          delete _this5.cancelFetch;
          delete _this5.continueFetch;
          delete _this5.isTransportCancelable; // End loop if currently paused

          continueLoop == null ? void 0 : continueLoop();
        };

        var resolve = function resolve(value) {
          done();
          outerResolve(value);
        };

        var reject = function reject(value) {
          done();
          outerReject(value);
        }; // Create callback to cancel this fetch


        _this5.cancelFetch = function (silent) {
          reject(new CancelledError(silent));
          cancelTransport == null ? void 0 : cancelTransport();
        }; // Create callback to continue this fetch


        _this5.continueFetch = function () {
          continueLoop == null ? void 0 : continueLoop();
        }; // Create loop function


        var run = _async(function () {
          return _catch(function () {
            // Execute query
            var promiseOrValue = fn(); // Check if the transport layer support cancellation

            if (isCancelable(promiseOrValue)) {
              cancelTransport = function cancelTransport() {
                try {
                  promiseOrValue.cancel();
                } catch (_unused) {}
              };

              _this5.isTransportCancelable = true;
            } // Await data


            return _await(promiseOrValue, function (data) {
              // Resolve with data
              resolve(data);
            });
          }, function (error) {
            var _options$retry, _options$retryDelay;

            // Stop if the fetch is already resolved
            if (resolved) {
              return;
            } // Do we need to retry the request?


            var failureCount = _this5.state.failureCount;
            var retry = (_options$retry = options.retry) != null ? _options$retry : 3;
            var retryDelay = (_options$retryDelay = options.retryDelay) != null ? _options$retryDelay : defaultRetryDelay;
            var shouldRetry = retry === true || failureCount < retry || typeof retry === 'function' && retry(failureCount, error);

            if (!shouldRetry) {
              // We are done if the query does not need to be retried
              reject(error);
              return;
            } // Increase the failureCount


            _this5.dispatch({
              type: 'failed'
            }); // Delay


            return _await(sleep(functionalUpdate(retryDelay, failureCount) || 0), function () {
              // Pause retry if the document is not visible or when the device is offline
              return _invoke(function () {
                if (!isDocumentVisible() || !isOnline()) {
                  return _awaitIgnored(new Promise(function (continueResolve) {
                    continueLoop = continueResolve;
                  }));
                }
              }, function () {
                if (!resolved) {
                  run();
                }
              }); // Try again if not resolved yet
            });
          });
        }); // Start loop


        run();
      });
    };

    return Query;
  }();

  function defaultQueryFn() {
    return Promise.reject();
  }

  function getLastPage(pages, previous) {
    return previous ? pages[0] : pages[pages.length - 1];
  }

  function hasMorePages(options, pages, previous) {
    if (options.infinite && options.getFetchMore && Array.isArray(pages)) {
      return Boolean(options.getFetchMore(getLastPage(pages, previous), pages));
    }
  }

  function getDefaultState(options) {
    var data = typeof options.initialData === 'function' ? options.initialData() : options.initialData;
    var hasData = typeof data !== 'undefined';
    return {
      canFetchMore: hasMorePages(options, data),
      data: data,
      dataUpdateCount: 0,
      error: null,
      errorUpdateCount: 0,
      failureCount: 0,
      isFetching: false,
      isFetchingMore: false,
      isInvalidated: false,
      status: hasData ? 'success' : 'idle',
      updatedAt: hasData ? Date.now() : 0
    };
  }

  function queryReducer(state, action) {
    var _action$updatedAt;

    switch (action.type) {
      case 'failed':
        return _extends({}, state, {
          failureCount: state.failureCount + 1
        });

      case 'fetch':
        return _extends({}, state, {
          failureCount: 0,
          isFetching: true,
          isFetchingMore: action.isFetchingMore || false,
          status: typeof state.data === 'undefined' ? 'loading' : 'success'
        });

      case 'success':
        return _extends({}, state, {
          canFetchMore: action.canFetchMore,
          data: action.data,
          dataUpdateCount: state.dataUpdateCount + 1,
          error: null,
          failureCount: 0,
          isFetching: false,
          isFetchingMore: false,
          isInvalidated: false,
          status: 'success',
          updatedAt: (_action$updatedAt = action.updatedAt) != null ? _action$updatedAt : Date.now()
        });

      case 'error':
        return _extends({}, state, {
          error: action.error,
          errorUpdateCount: state.errorUpdateCount + 1,
          failureCount: state.failureCount + 1,
          isFetching: false,
          isFetchingMore: false,
          status: 'error'
        });

      case 'invalidate':
        return _extends({}, state, {
          isInvalidated: true
        });

      default:
        return state;
    }
  }

  // CLASS
  var QueryCache = /*#__PURE__*/function () {
    function QueryCache() {
      this.listeners = [];
      this.queries = [];
      this.queriesMap = {};
    }

    var _proto = QueryCache.prototype;

    _proto.build = function build(options) {
      var queryKey = options.queryKey;
      var queryHash = options.queryHash || hashQueryKey(queryKey, options);
      var query = this.get(queryHash);

      if (!query) {
        query = new Query({
          cache: this,
          queryKey: queryKey,
          queryHash: queryHash,
          options: options
        });
        this.add(query);
      }

      return query;
    };

    _proto.add = function add(query) {
      if (!this.queriesMap[query.queryHash]) {
        this.queriesMap[query.queryHash] = query;
        this.queries.push(query);
        this.notify(query);
      }
    };

    _proto.remove = function remove(query) {
      if (this.queriesMap[query.queryHash]) {
        query.destroy();
        delete this.queriesMap[query.queryHash];
        this.queries = this.queries.filter(function (x) {
          return x !== query;
        });
        this.notify(query);
      }
    };

    _proto.clear = function clear() {
      var _this = this;

      notifyManager.batch(function () {
        _this.queries.forEach(function (query) {
          _this.remove(query);
        });
      });
    };

    _proto.get = function get(queryHash) {
      return this.queriesMap[queryHash];
    };

    _proto.getAll = function getAll() {
      return this.queries;
    };

    _proto.find = function find(arg1, arg2) {
      var _parseFilterArgs = parseFilterArgs(arg1, arg2),
          filters = _parseFilterArgs[0];

      return this.queries.find(function (query) {
        return matchQuery(filters, query);
      });
    };

    _proto.findAll = function findAll(arg1, arg2) {
      var _parseFilterArgs2 = parseFilterArgs(arg1, arg2),
          filters = _parseFilterArgs2[0];

      return filters && Object.keys(filters).length > 0 ? this.queries.filter(function (query) {
        return matchQuery(filters, query);
      }) : this.queries;
    };

    _proto.subscribe = function subscribe(listener) {
      var _this2 = this;

      this.listeners.push(listener);
      return function () {
        _this2.listeners = _this2.listeners.filter(function (x) {
          return x !== listener;
        });
      };
    };

    _proto.notify = function notify(query) {
      var _this3 = this;

      notifyManager.batch(function () {
        _this3.listeners.forEach(function (listener) {
          notifyManager.schedule(function () {
            listener(query);
          });
        });
      });
    };

    return QueryCache;
  }();

  var QueriesObserver = /*#__PURE__*/function () {
    function QueriesObserver(config) {
      this.client = config.client;
      this.queries = config.queries || [];
      this.result = [];
      this.observers = [];
      this.listeners = []; // Subscribe to queries

      this.updateObservers();
    }

    var _proto = QueriesObserver.prototype;

    _proto.subscribe = function subscribe(listener) {
      var _this = this;

      var callback = listener || function () {
        return undefined;
      };

      this.listeners.push(callback);

      if (this.listeners.length === 1) {
        this.onMount();
      }

      return function () {
        _this.unsubscribe(callback);
      };
    };

    _proto.unsubscribe = function unsubscribe(listener) {
      this.listeners = this.listeners.filter(function (x) {
        return x !== listener;
      });

      if (!this.listeners.length) {
        this.clear();
      }
    };

    _proto.onMount = function onMount() {
      var _this2 = this;

      this.observers.forEach(function (observer) {
        observer.subscribe(function (result) {
          _this2.onUpdate(observer, result);
        });
      });
    };

    _proto.clear = function clear() {
      this.listeners = [];
      this.observers.forEach(function (observer) {
        observer.clear();
      });
    };

    _proto.setQueries = function setQueries(queries) {
      this.queries = queries;
      this.updateObservers();
    };

    _proto.getCurrentResult = function getCurrentResult() {
      return this.result;
    };

    _proto.updateObservers = function updateObservers() {
      var _this3 = this;

      var hasIndexChange = false;
      var prevObservers = this.observers;
      var newObservers = this.queries.map(function (options, i) {
        var _observer;

        var observer = prevObservers[i];

        var defaultedOptions = _this3.client.defaultQueryObserverOptions(options);

        defaultedOptions.queryHash = hashQueryKey(defaultedOptions.queryKey, defaultedOptions);

        if (((_observer = observer) == null ? void 0 : _observer.getCurrentQuery().queryHash) !== defaultedOptions.queryHash) {
          hasIndexChange = true;
          observer = prevObservers.find(function (x) {
            return x.getCurrentQuery().queryHash === defaultedOptions.queryHash;
          });
        }

        if (observer) {
          observer.setOptions(defaultedOptions);
          return observer;
        }

        return _this3.client.watchQuery(defaultedOptions);
      });

      if (prevObservers.length === newObservers.length && !hasIndexChange) {
        return;
      }

      this.observers = newObservers;
      this.result = newObservers.map(function (observer) {
        return observer.getCurrentResult();
      });

      if (!this.listeners.length) {
        return;
      }

      difference(prevObservers, newObservers).forEach(function (observer) {
        observer.clear();
      });
      difference(newObservers, prevObservers).forEach(function (observer) {
        observer.subscribe(function (result) {
          _this3.onUpdate(observer, result);
        });
      });
      this.notify();
    };

    _proto.onUpdate = function onUpdate(observer, result) {
      var index = this.observers.indexOf(observer);

      if (index !== -1) {
        this.result = replaceAt(this.result, index, result);
        this.notify();
      }
    };

    _proto.notify = function notify() {
      var result = this.result,
          listeners = this.listeners;
      listeners.forEach(function (listener) {
        notifyManager.schedule(function () {
          listener(result);
        });
      });
    };

    return QueriesObserver;
  }();

  var QueryObserver = /*#__PURE__*/function () {
    function QueryObserver(config) {
      this.client = config.client;
      this.options = config.client.defaultQueryObserverOptions(config.options);
      this.listeners = [];
      this.initialDataUpdateCount = 0; // Bind exposed methods

      this.remove = this.remove.bind(this);
      this.refetch = this.refetch.bind(this);
      this.fetchMore = this.fetchMore.bind(this); // Subscribe to the query

      this.updateQuery();
    }

    var _proto = QueryObserver.prototype;

    _proto.subscribe = function subscribe(listener) {
      var _this = this;

      var callback = listener || function () {
        return undefined;
      };

      this.listeners.push(callback);

      if (this.listeners.length === 1) {
        this.onMount();
      }

      return function () {
        _this.unsubscribe(callback);
      };
    };

    _proto.unsubscribe = function unsubscribe(listener) {
      this.listeners = this.listeners.filter(function (x) {
        return x !== listener;
      });

      if (!this.listeners.length) {
        this.clear();
      }
    };

    _proto.onMount = function onMount() {
      this.currentQuery.subscribeObserver(this);

      if (this.willFetchOnMount()) {
        this.executeFetch();
      }

      this.updateTimers();
    };

    _proto.willFetchOnMount = function willFetchOnMount() {
      return this.options.enabled !== false && (!this.currentQuery.state.updatedAt || this.options.refetchOnMount === 'always' || this.options.refetchOnMount !== false && this.isStale());
    };

    _proto.willFetchOptionally = function willFetchOptionally() {
      return this.options.enabled !== false && this.isStale();
    };

    _proto.isStale = function isStale() {
      return this.currentQuery.isStaleByTime(this.options.staleTime);
    };

    _proto.clear = function clear() {
      this.listeners = [];
      this.clearTimers();
      this.currentQuery.unsubscribeObserver(this);
    };

    _proto.setOptions = function setOptions(options) {
      var prevOptions = this.options;
      var prevQuery = this.currentQuery;
      this.options = this.client.defaultQueryObserverOptions(options); // Keep previous query key if the user does not supply one

      if (!this.options.queryKey) {
        this.options.queryKey = prevOptions.queryKey;
      }

      this.updateQuery(); // Take no further actions if there are no subscribers

      if (!this.listeners.length) {
        return;
      } // If we subscribed to a new query, optionally fetch and update refetch


      if (this.currentQuery !== prevQuery) {
        this.optionalFetch();
        this.updateTimers();
        return;
      } // Optionally fetch if the query became enabled


      if (this.options.enabled !== false && prevOptions.enabled === false) {
        this.optionalFetch();
      } // Update stale interval if needed


      if (this.options.enabled !== prevOptions.enabled || this.options.staleTime !== prevOptions.staleTime) {
        this.updateStaleTimeout();
      } // Update refetch interval if needed


      if (this.options.enabled !== prevOptions.enabled || this.options.refetchInterval !== prevOptions.refetchInterval) {
        this.updateRefetchInterval();
      }
    };

    _proto.getCurrentResult = function getCurrentResult() {
      return this.currentResult;
    };

    _proto.getCurrentOrNextResult = function getCurrentOrNextResult(options) {
      if (this.currentQuery.isFetching()) {
        return this.getNextResult(options);
      } else if (this.currentResult.isError && (options == null ? void 0 : options.throwOnError)) {
        return Promise.reject(this.currentResult.error);
      }

      return Promise.resolve(this.currentResult);
    };

    _proto.getNextResult = function getNextResult(options) {
      var _this2 = this;

      return new Promise(function (resolve, reject) {
        var unsubscribe = _this2.subscribe(function (result) {
          if (!result.isFetching) {
            unsubscribe();

            if (result.isError && (options == null ? void 0 : options.throwOnError)) {
              reject(result.error);
            } else {
              resolve(result);
            }
          }
        });
      });
    };

    _proto.getCurrentQuery = function getCurrentQuery() {
      return this.currentQuery;
    };

    _proto.remove = function remove() {
      this.currentQuery.remove();
    };

    _proto.refetch = function refetch(options) {
      return this.fetch(options);
    };

    _proto.fetchMore = function fetchMore(fetchMoreVariable, options) {
      return this.fetch({
        throwOnError: options == null ? void 0 : options.throwOnError,
        fetchMore: {
          previous: options == null ? void 0 : options.previous,
          fetchMoreVariable: fetchMoreVariable
        }
      });
    };

    _proto.fetch = function fetch(fetchOptions) {
      if (!this.canFetch()) {
        return this.getCurrentOrNextResult();
      }

      var promise = this.getNextResult(fetchOptions);
      this.executeFetch(fetchOptions);
      return promise;
    };

    _proto.optionalFetch = function optionalFetch() {
      if (this.willFetchOptionally()) {
        this.executeFetch();
      }
    };

    _proto.executeFetch = function executeFetch(fetchOptions) {
      if (this.canFetch()) {
        this.currentQuery.fetch(this.getQueryOptions(), fetchOptions).catch(noop);
      }
    };

    _proto.canFetch = function canFetch() {
      var _this$currentQuery$de;

      return Boolean(this.options.queryFn || ((_this$currentQuery$de = this.currentQuery.defaultOptions) == null ? void 0 : _this$currentQuery$de.queryFn));
    };

    _proto.updateStaleTimeout = function updateStaleTimeout() {
      var _this3 = this;

      this.clearStaleTimeout();

      if (isServer || this.currentResult.isStale || !isValidTimeout(this.options.staleTime)) {
        return;
      }

      var time = timeUntilStale(this.currentResult.updatedAt, this.options.staleTime); // The timeout is sometimes triggered 1 ms before the stale time expiration.
      // To mitigate this issue we always add 1 ms to the timeout.

      var timeout = time + 1;
      this.staleTimeoutId = setTimeout(function () {
        if (!_this3.currentResult.isStale) {
          _this3.updateResult();

          _this3.notify({
            listeners: true,
            cache: true
          });
        }
      }, timeout);
    };

    _proto.updateRefetchInterval = function updateRefetchInterval() {
      var _this4 = this;

      this.clearRefetchInterval();

      if (isServer || this.options.enabled === false || !isValidTimeout(this.options.refetchInterval)) {
        return;
      }

      this.refetchIntervalId = setInterval(function () {
        if (_this4.options.refetchIntervalInBackground || isDocumentVisible()) {
          _this4.executeFetch();
        }
      }, this.options.refetchInterval);
    };

    _proto.updateTimers = function updateTimers() {
      this.updateStaleTimeout();
      this.updateRefetchInterval();
    };

    _proto.clearTimers = function clearTimers() {
      this.clearStaleTimeout();
      this.clearRefetchInterval();
    };

    _proto.clearStaleTimeout = function clearStaleTimeout() {
      clearInterval(this.staleTimeoutId);
      this.staleTimeoutId = undefined;
    };

    _proto.clearRefetchInterval = function clearRefetchInterval() {
      clearInterval(this.refetchIntervalId);
      this.refetchIntervalId = undefined;
    };

    _proto.getQueryOptions = function getQueryOptions() {
      return this.options;
    };

    _proto.updateResult = function updateResult(willFetch) {
      var _this$previousQueryRe;

      var state = this.currentQuery.state;
      var status = state.status,
          isFetching = state.isFetching,
          updatedAt = state.updatedAt;
      var isPreviousData = false;
      var data; // Already set the status to loading if we are going to fetch

      if (willFetch) {
        isFetching = true;

        if (status === 'idle') {
          status = 'loading';
        }
      } // Keep previous data if needed


      if (this.options.keepPreviousData && !state.dataUpdateCount && ((_this$previousQueryRe = this.previousQueryResult) == null ? void 0 : _this$previousQueryRe.isSuccess)) {
        data = this.previousQueryResult.data;
        updatedAt = this.previousQueryResult.updatedAt;
        status = this.previousQueryResult.status;
        isPreviousData = true;
      } else if (this.options.select && typeof state.data !== 'undefined') {
        var _this$currentState;

        // Use the previous select result if the query data did not change
        if (this.currentResult && state.data === ((_this$currentState = this.currentState) == null ? void 0 : _this$currentState.data)) {
          data = this.currentResult.data;
        } else {
          data = this.options.select(state.data);
        }
      } else {
        data = state.data;
      }

      var result = _extends({}, getStatusProps(status), {
        canFetchMore: state.canFetchMore,
        data: data,
        error: state.error,
        failureCount: state.failureCount,
        fetchMore: this.fetchMore,
        isFetched: state.dataUpdateCount > 0,
        isFetchedAfterMount: state.dataUpdateCount > this.initialDataUpdateCount,
        isFetching: isFetching,
        isFetchingMore: state.isFetchingMore,
        isPreviousData: isPreviousData,
        isStale: this.isStale(),
        refetch: this.refetch,
        remove: this.remove,
        updatedAt: updatedAt
      }); // Keep reference to the current state on which the current result is based on


      this.currentState = state; // Only update if something has changed

      if (!this.currentResult || !shallowEqualObjects(this.currentResult, result)) {
        this.currentResult = result;
      }
    };

    _proto.updateQuery = function updateQuery() {
      var prevQuery = this.currentQuery;
      var queryOptions = this.getQueryOptions();
      var query = this.client.getCache().build(queryOptions);

      if (query === prevQuery) {
        return;
      }

      this.previousQueryResult = this.currentResult;
      this.currentQuery = query;
      this.initialDataUpdateCount = query.state.dataUpdateCount;
      var willFetch = prevQuery ? this.willFetchOptionally() : this.willFetchOnMount();
      this.updateResult(willFetch);

      if (!this.listeners.length) {
        return;
      }

      prevQuery == null ? void 0 : prevQuery.unsubscribeObserver(this);
      this.currentQuery.subscribeObserver(this);

      if (this.options.notifyOnStatusChange !== false) {
        this.notify({
          listeners: true
        });
      }
    };

    _proto.onQueryUpdate = function onQueryUpdate(action) {
      var options = this.options;
      var type = action.type; // Store current result and get new result

      var prevResult = this.currentResult;
      this.updateResult();
      var currentResult = this.currentResult; // Update timers if needed

      if (type === 'success' || type === 'error' || type === 'invalidate') {
        this.updateTimers();
      } // Do not notify if the nothing has changed


      if (prevResult === currentResult) {
        return;
      } // Determine which callbacks to trigger


      var notifyOptions = {};

      if (type === 'success') {
        notifyOptions.onSuccess = true;
      } else if (type === 'error') {
        notifyOptions.onError = true;
      }

      if ( // Always notify if notifyOnStatusChange is set
      options.notifyOnStatusChange !== false || // Otherwise only notify on data or error change
      currentResult.data !== prevResult.data || currentResult.error !== prevResult.error) {
        notifyOptions.listeners = true;
      }

      this.notify(notifyOptions);
    };

    _proto.notify = function notify(notifyOptions) {
      var _this5 = this;

      var options = this.options,
          currentResult = this.currentResult,
          currentQuery = this.currentQuery,
          listeners = this.listeners;
      var onSuccess = options.onSuccess,
          onSettled = options.onSettled,
          onError = options.onError;
      notifyManager.batch(function () {
        // First trigger the configuration callbacks
        if (notifyOptions.onSuccess) {
          if (onSuccess) {
            notifyManager.schedule(function () {
              onSuccess(currentResult.data);
            });
          }

          if (onSettled) {
            notifyManager.schedule(function () {
              onSettled(currentResult.data, null);
            });
          }
        } else if (notifyOptions.onError) {
          if (onError) {
            notifyManager.schedule(function () {
              onError(currentResult.error);
            });
          }

          if (onSettled) {
            notifyManager.schedule(function () {
              onSettled(undefined, currentResult.error);
            });
          }
        } // Then trigger the listeners


        if (notifyOptions.listeners) {
          listeners.forEach(function (listener) {
            notifyManager.schedule(function () {
              listener(currentResult);
            });
          });
        } // Then the cache listeners


        if (notifyOptions.cache) {
          _this5.client.getCache().notify(currentQuery);
        }
      });
    };

    return QueryObserver;
  }();

  // CLASS
  var QueryClient = /*#__PURE__*/function () {
    function QueryClient(config) {
      this.cache = config.cache;
      this.defaultOptions = config.defaultOptions || {};
    }

    var _proto = QueryClient.prototype;

    _proto.getDefaultOptions = function getDefaultOptions() {
      return this.defaultOptions;
    };

    _proto.setDefaultOptions = function setDefaultOptions(options) {
      this.defaultOptions = options;
    };

    _proto.mount = function mount() {
      mountedClients.push(this);
    };

    _proto.unmount = function unmount() {
      var index = mountedClients.indexOf(this);

      if (index > -1) {
        mountedClients.splice(index, 1);
      }
    };

    _proto.isFetching = function isFetching(arg1, arg2) {
      var _parseFilterArgs = parseFilterArgs(arg1, arg2),
          filters = _parseFilterArgs[0];

      filters.fetching = true;
      return this.cache.findAll(filters).length;
    };

    _proto.setQueryDefaults = function setQueryDefaults(arg1, arg2, arg3) {
      var parsedOptions = parseQueryArgs(arg1, arg2, arg3);
      var defaultedOptions = this.defaultQueryOptions(parsedOptions);
      this.cache.build(defaultedOptions).setDefaultOptions(defaultedOptions);
    };

    _proto.getQueryData = function getQueryData(queryKey, filters) {
      var _this$cache$find;

      return (_this$cache$find = this.cache.find(queryKey, filters)) == null ? void 0 : _this$cache$find.state.data;
    };

    _proto.setQueryData = function setQueryData(queryKey, updater) {
      var parsedOptions = parseQueryArgs(queryKey);
      var defaultedOptions = this.defaultQueryOptions(parsedOptions);
      return this.cache.build(defaultedOptions).setData(updater);
    };

    _proto.getQueryState = function getQueryState(queryKey, filters) {
      var _this$cache$find2;

      return (_this$cache$find2 = this.cache.find(queryKey, filters)) == null ? void 0 : _this$cache$find2.state;
    };

    _proto.removeQueries = function removeQueries(arg1, arg2) {
      var _this = this;

      notifyManager.batch(function () {
        _this.cache.findAll(arg1, arg2).forEach(function (query) {
          _this.cache.remove(query);
        });
      });
    };

    _proto.cancelQueries = function cancelQueries(arg1, arg2) {
      var _this2 = this;

      var promises = notifyManager.batch(function () {
        return _this2.cache.findAll(arg1, arg2).map(function (query) {
          return query.cancel();
        });
      });
      return Promise.all(promises).then(noop);
    };

    _proto.invalidateQueries = function invalidateQueries(arg1, arg2, arg3) {
      var _filters$refetchActiv,
          _filters$refetchInact,
          _this3 = this;

      var _parseFilterArgs2 = parseFilterArgs(arg1, arg2, arg3),
          filters = _parseFilterArgs2[0],
          options = _parseFilterArgs2[1];

      var refetchFilters = _extends({}, filters, {
        active: (_filters$refetchActiv = filters.refetchActive) != null ? _filters$refetchActiv : true,
        inactive: (_filters$refetchInact = filters.refetchInactive) != null ? _filters$refetchInact : false
      });

      return notifyManager.batch(function () {
        _this3.cache.findAll(filters).forEach(function (query) {
          query.invalidate();
        });

        return _this3.refetchQueries(refetchFilters, options);
      });
    };

    _proto.refetchQueries = function refetchQueries(arg1, arg2, arg3) {
      var _this4 = this;

      var _parseFilterArgs3 = parseFilterArgs(arg1, arg2, arg3),
          filters = _parseFilterArgs3[0],
          options = _parseFilterArgs3[1];

      var promises = notifyManager.batch(function () {
        return _this4.cache.findAll(filters).map(function (query) {
          return query.fetch();
        });
      });
      var promise = Promise.all(promises).then(noop);

      if (!(options == null ? void 0 : options.throwOnError)) {
        promise = promise.catch(noop);
      }

      return promise;
    };

    _proto.watchQuery = function watchQuery(arg1, arg2, arg3) {
      var parsedOptions = parseQueryArgs(arg1, arg2, arg3);
      return new QueryObserver({
        client: this,
        options: parsedOptions
      });
    };

    _proto.watchQueries = function watchQueries(queries) {
      return new QueriesObserver({
        client: this,
        queries: queries
      });
    };

    _proto.fetchQueryData = function fetchQueryData(arg1, arg2, arg3) {
      var parsedOptions = parseQueryArgs(arg1, arg2, arg3); // https://github.com/tannerlinsley/react-query/issues/652

      if (typeof parsedOptions.retry === 'undefined') {
        parsedOptions.retry = false;
      }

      var defaultedOptions = this.defaultQueryOptions(parsedOptions);
      var query = this.cache.find(defaultedOptions.queryKey);

      if (!query) {
        query = this.cache.build(defaultedOptions);
      } else if (!query.isStaleByTime(defaultedOptions.staleTime)) {
        return Promise.resolve(query.state.data);
      }

      return query.fetch(defaultedOptions);
    };

    _proto.prefetchQuery = function prefetchQuery(arg1, arg2, arg3) {
      return this.fetchQueryData(arg1, arg2, arg3).then(noop).catch(noop);
    };

    _proto.getCache = function getCache() {
      return this.cache;
    };

    _proto.defaultQueryOptions = function defaultQueryOptions(options) {
      return _extends({}, this.defaultOptions.queries, options);
    };

    _proto.defaultQueryObserverOptions = function defaultQueryObserverOptions(options) {
      return _extends({}, this.defaultOptions.queries, options);
    };

    _proto.defaultMutationOptions = function defaultMutationOptions(options) {
      return _extends({}, this.defaultOptions.mutations, options);
    };

    return QueryClient;
  }();
  var mountedClients = [];
  function onExternalEvent(type) {
    if (isDocumentVisible() && isOnline()) {
      notifyManager.batch(function () {
        uniq(mountedClients.map(function (x) {
          return x.getCache();
        })).forEach(function (cache) {
          cache.getAll().forEach(function (query) {
            if (type === 'focus') {
              query.onFocus();
            } else {
              query.onOnline();
            }
          });
        });
      });
    }
  }

  var setFocusHandler = createSetHandler(function () {
    return onExternalEvent('focus');
  });
  setFocusHandler(function (handleFocus) {
    var _window;

    if (isServer || !((_window = window) == null ? void 0 : _window.addEventListener)) {
      return;
    } // Listen to visibillitychange and focus


    window.addEventListener('visibilitychange', handleFocus, false);
    window.addEventListener('focus', handleFocus, false);
    return function () {
      // Be sure to unsubscribe if a new handler is set
      window.removeEventListener('visibilitychange', handleFocus);
      window.removeEventListener('focus', handleFocus);
    };
  });

  var setOnlineHandler = createSetHandler(function () {
    return onExternalEvent('online');
  });
  setOnlineHandler(function (handleOnline) {
    var _window;

    if (isServer || !((_window = window) == null ? void 0 : _window.addEventListener)) {
      return;
    } // Listen to online


    window.addEventListener('online', handleOnline, false);
    return function () {
      // Be sure to unsubscribe if a new handler is set
      window.removeEventListener('online', handleOnline);
    };
  });

  var Console = console;

  var unstable_batchedUpdates = ReactDOM.unstable_batchedUpdates;

  var queryClientContext = /*#__PURE__*/React.createContext(undefined);
  var useQueryClient = function useQueryClient() {
    return React.useContext(queryClientContext);
  };
  var QueryClientProvider = function QueryClientProvider(_ref) {
    var client = _ref.client,
        children = _ref.children;
    React.useEffect(function () {
      client.mount();
      return function () {
        client.unmount();
      };
    }, [client]);
    return /*#__PURE__*/React.createElement(queryClientContext.Provider, {
      value: client
    }, children);
  };

  function createValue() {
    var _isReset = false;
    return {
      clearReset: function clearReset() {
        _isReset = false;
      },
      reset: function reset() {
        _isReset = true;
      },
      isReset: function isReset() {
        return _isReset;
      }
    };
  }

  var context = /*#__PURE__*/React.createContext(createValue()); // HOOK

  var useQueryErrorResetBoundary = function useQueryErrorResetBoundary() {
    return React.useContext(context);
  }; // COMPONENT

  var QueryErrorResetBoundary = function QueryErrorResetBoundary(_ref) {
    var children = _ref.children;
    var value = React.useMemo(function () {
      return createValue();
    }, []);
    return /*#__PURE__*/React.createElement(context.Provider, {
      value: value
    }, typeof children === 'function' ? children(value) : children);
  };

  function useIsMounted() {
    var mountedRef = React.useRef(false);
    var isMounted = React.useCallback(function () {
      return mountedRef.current;
    }, []);
    var useEffect = isServer ? React.useEffect : React.useLayoutEffect;
    useEffect(function () {
      mountedRef.current = true;
      return function () {
        mountedRef.current = false;
      };
    }, []);
    return isMounted;
  }
  function useMountedCallback(callback) {
    var isMounted = useIsMounted();
    return React.useCallback(function () {
      if (isMounted()) {
        return callback.apply(void 0, arguments);
      }
    }, [callback, isMounted]);
  }

  function useIsFetching(arg1, arg2) {
    var client = useQueryClient();
    var isMounted = useIsMounted();

    var _parseFilterArgs = parseFilterArgs(arg1, arg2),
        filters = _parseFilterArgs[0];

    var _React$useState = React.useState(client.isFetching(filters)),
        isFetching = _React$useState[0],
        setIsFetching = _React$useState[1];

    var filtersRef = React.useRef(filters);
    filtersRef.current = filters;
    var isFetchingRef = React.useRef(isFetching);
    isFetchingRef.current = isFetching;
    React.useEffect(function () {
      return client.getCache().subscribe(function () {
        if (isMounted()) {
          var newIsFetching = client.isFetching(filtersRef.current);

          if (isFetchingRef.current !== newIsFetching) {
            setIsFetching(newIsFetching);
          }
        }
      });
    }, [client, setIsFetching, isMounted]);
    return isFetching;
  }

  // HOOK
  function _await$1(value, then, direct) {
    if (direct) {
      return then ? then(value) : value;
    }

    if (!value || !value.then) {
      value = Promise.resolve(value);
    }

    return then ? value.then(then) : value;
  }

  function _catch$1(body, recover) {
    try {
      var result = body();
    } catch (e) {
      return recover(e);
    }

    if (result && result.then) {
      return result.then(void 0, recover);
    }

    return result;
  }

  function _async$1(f) {
    return function () {
      for (var args = [], i = 0; i < arguments.length; i++) {
        args[i] = arguments[i];
      }

      try {
        return Promise.resolve(f.apply(this, args));
      } catch (e) {
        return Promise.reject(e);
      }
    };
  }

  function getDefaultState$1() {
    return _extends({}, getStatusProps('idle'), {
      data: undefined,
      error: null
    });
  }

  function reducer(state, action) {
    switch (action.type) {
      case 'reset':
        return getDefaultState$1();

      case 'loading':
        return _extends({}, getStatusProps('loading'), {
          data: undefined,
          error: null
        });

      case 'success':
        return _extends({}, getStatusProps('success'), {
          data: action.data,
          error: null
        });

      case 'error':
        return _extends({}, getStatusProps('error'), {
          data: undefined,
          error: action.error
        });

      default:
        return state;
    }
  }

  function useMutation(mutationFn, options) {
    if (options === void 0) {
      options = {};
    }

    var _React$useReducer = React.useReducer(reducer, null, getDefaultState$1),
        state = _React$useReducer[0],
        unsafeDispatch = _React$useReducer[1];

    var dispatch = useMountedCallback(unsafeDispatch);
    var client = useQueryClient();
    var defaultedOptions = client.defaultMutationOptions(options);
    var latestMutationRef = React.useRef(0);
    var latestMutationFnRef = React.useRef(mutationFn);
    latestMutationFnRef.current = mutationFn;
    var latestOptionsRef = React.useRef(defaultedOptions);
    latestOptionsRef.current = defaultedOptions;
    var mutate = React.useCallback(_async$1(function (variables, mutateOptions) {
      if (mutateOptions === void 0) {
        mutateOptions = {};
      }

      dispatch({
        type: 'loading'
      });
      var mutationId = ++latestMutationRef.current;
      var latestOptions = latestOptionsRef.current;
      var latestMutationFn = latestMutationFnRef.current;

      var isLatest = function isLatest() {
        return latestMutationRef.current === mutationId;
      };

      var context;
      return _catch$1(function () {
        return _await$1(latestOptions.onMutate == null ? void 0 : latestOptions.onMutate(variables), function (_latestOptions$onMuta) {
          context = _latestOptions$onMuta;
          return _await$1(latestMutationFn(variables), function (data) {
            return _await$1(latestOptions.onSuccess == null ? void 0 : latestOptions.onSuccess(data, variables, context), function () {
              return _await$1(mutateOptions.onSuccess == null ? void 0 : mutateOptions.onSuccess(data, variables, context), function () {
                return _await$1(latestOptions.onSettled == null ? void 0 : latestOptions.onSettled(data, null, variables, context), function () {
                  return _await$1(mutateOptions.onSettled == null ? void 0 : mutateOptions.onSettled(data, null, variables, context), function () {
                    // Dispatch after handlers as the mutation could still fail
                    if (isLatest()) {
                      dispatch({
                        type: 'success',
                        data: data
                      });
                    }

                    return data;
                  });
                });
              });
            });
          });
        });
      }, function (error) {
        getConsole().error(error);
        return _await$1(latestOptions.onError == null ? void 0 : latestOptions.onError(error, variables, context), function () {
          return _await$1(mutateOptions.onError == null ? void 0 : mutateOptions.onError(error, variables, context), function () {
            return _await$1(latestOptions.onSettled == null ? void 0 : latestOptions.onSettled(undefined, error, variables, context), function () {
              return _await$1(mutateOptions.onSettled == null ? void 0 : mutateOptions.onSettled(undefined, error, variables, context), function () {
                if (isLatest()) {
                  dispatch({
                    type: 'error',
                    error: error
                  });
                }

                if (mutateOptions.throwOnError || latestOptions.throwOnError) {
                  throw error;
                }
              });
            });
          });
        });
      });
    }), [dispatch]);
    React.useEffect(function () {
      var latestOptions = latestOptionsRef.current;

      if (state.error && (latestOptions.useErrorBoundary || latestOptions.suspense)) {
        throw state.error;
      }
    }, [state.error]);
    var reset = React.useCallback(function () {
      dispatch({
        type: 'reset'
      });
    }, [dispatch]);
    return React.useMemo(function () {
      return [mutate, _extends({}, state, {
        reset: reset
      })];
    }, [mutate, state, reset]);
  }

  function useBaseQuery(options) {
    var client = useQueryClient();
    var isMounted = useIsMounted();
    var errorResetBoundary = useQueryErrorResetBoundary();
    var defaultedOptions = client.defaultQueryObserverOptions(options); // Always set stale time when using suspense

    if (defaultedOptions.suspense && !defaultedOptions.staleTime) {
      defaultedOptions.staleTime = 5000;
    } // Create query observer


    var observerRef = React.useRef();
    var firstRender = !observerRef.current;
    var observer = observerRef.current || client.watchQuery(defaultedOptions);
    observerRef.current = observer; // Update options

    if (!firstRender) {
      observer.setOptions(defaultedOptions);
    }

    var _React$useState = React.useState(function () {
      return observer.getCurrentResult();
    }),
        currentResult = _React$useState[0],
        setCurrentResult = _React$useState[1]; // Subscribe to the observer


    React.useEffect(function () {
      errorResetBoundary.clearReset();
      return observer.subscribe(function (result) {
        if (isMounted()) {
          setCurrentResult(result);
        }
      });
    }, [isMounted, observer, errorResetBoundary]); // Handle suspense

    if (observer.options.suspense || observer.options.useErrorBoundary) {
      if (currentResult.isError && !errorResetBoundary.isReset()) {
        throw currentResult.error;
      }

      if (observer.options.suspense && firstRender && observer.willFetchOnMount()) {
        errorResetBoundary.clearReset();
        throw observer.getNextResult({
          throwOnError: true
        });
      }
    }

    return currentResult;
  }

  function useQuery(arg1, arg2, arg3) {
    var parsedOptions = parseQueryArgs(arg1, arg2, arg3);
    return useBaseQuery(parsedOptions);
  }

  function useQueries(queries) {
    var client = useQueryClient();
    var isMounted = useIsMounted(); // Create queries observer

    var observerRef = React.useRef();
    var firstRender = !observerRef.current;
    var observer = observerRef.current || client.watchQueries(queries);
    observerRef.current = observer; // Update queries

    if (!firstRender) {
      observer.setQueries(queries);
    }

    var _React$useState = React.useState(function () {
      return observer.getCurrentResult();
    }),
        currentResult = _React$useState[0],
        setCurrentResult = _React$useState[1]; // Subscribe to the observer


    React.useEffect(function () {
      return observer.subscribe(function (result) {
        if (isMounted()) {
          setCurrentResult(result);
        }
      });
    }, [isMounted, observer, setCurrentResult]);
    return currentResult;
  }

  function useInfiniteQuery(arg1, arg2, arg3) {
    var parsedOptions = parseQueryArgs(arg1, arg2, arg3);
    parsedOptions.infinite = true;
    return useBaseQuery(parsedOptions);
  }

  setBatchedUpdates(unstable_batchedUpdates);

  if (Console) {
    setConsole(Console);
  }

  exports.CancelledError = CancelledError;
  exports.Query = Query;
  exports.QueryCache = QueryCache;
  exports.QueryClient = QueryClient;
  exports.QueryClientProvider = QueryClientProvider;
  exports.QueryErrorResetBoundary = QueryErrorResetBoundary;
  exports.defaultQueryKeySerializerFn = defaultQueryKeySerializerFn;
  exports.isCancelledError = isCancelledError;
  exports.isError = isError;
  exports.setBatchedUpdates = setBatchedUpdates;
  exports.setConsole = setConsole;
  exports.setFocusHandler = setFocusHandler;
  exports.setOnlineHandler = setOnlineHandler;
  exports.useInfiniteQuery = useInfiniteQuery;
  exports.useIsFetching = useIsFetching;
  exports.useMutation = useMutation;
  exports.useQueries = useQueries;
  exports.useQuery = useQuery;
  exports.useQueryClient = useQueryClient;
  exports.useQueryErrorResetBoundary = useQueryErrorResetBoundary;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=react-query.development.js.map
