import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import { useMountedCallback } from './utils';
import { getStatusProps } from '../core/utils';
import { getConsole } from '../core/setConsole';
import { useQueryClient } from './QueryClientProvider';

// HOOK
function _await(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

function _catch(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
}

function _async(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

function getDefaultState() {
  return _extends({}, getStatusProps('idle'), {
    data: undefined,
    error: null
  });
}

function reducer(state, action) {
  switch (action.type) {
    case 'reset':
      return getDefaultState();

    case 'loading':
      return _extends({}, getStatusProps('loading'), {
        data: undefined,
        error: null
      });

    case 'success':
      return _extends({}, getStatusProps('success'), {
        data: action.data,
        error: null
      });

    case 'error':
      return _extends({}, getStatusProps('error'), {
        data: undefined,
        error: action.error
      });

    default:
      return state;
  }
}

export function useMutation(mutationFn, options) {
  if (options === void 0) {
    options = {};
  }

  var _React$useReducer = React.useReducer(reducer, null, getDefaultState),
      state = _React$useReducer[0],
      unsafeDispatch = _React$useReducer[1];

  var dispatch = useMountedCallback(unsafeDispatch);
  var client = useQueryClient();
  var defaultedOptions = client.defaultMutationOptions(options);
  var latestMutationRef = React.useRef(0);
  var latestMutationFnRef = React.useRef(mutationFn);
  latestMutationFnRef.current = mutationFn;
  var latestOptionsRef = React.useRef(defaultedOptions);
  latestOptionsRef.current = defaultedOptions;
  var mutate = React.useCallback(_async(function (variables, mutateOptions) {
    if (mutateOptions === void 0) {
      mutateOptions = {};
    }

    dispatch({
      type: 'loading'
    });
    var mutationId = ++latestMutationRef.current;
    var latestOptions = latestOptionsRef.current;
    var latestMutationFn = latestMutationFnRef.current;

    var isLatest = function isLatest() {
      return latestMutationRef.current === mutationId;
    };

    var context;
    return _catch(function () {
      return _await(latestOptions.onMutate == null ? void 0 : latestOptions.onMutate(variables), function (_latestOptions$onMuta) {
        context = _latestOptions$onMuta;
        return _await(latestMutationFn(variables), function (data) {
          return _await(latestOptions.onSuccess == null ? void 0 : latestOptions.onSuccess(data, variables, context), function () {
            return _await(mutateOptions.onSuccess == null ? void 0 : mutateOptions.onSuccess(data, variables, context), function () {
              return _await(latestOptions.onSettled == null ? void 0 : latestOptions.onSettled(data, null, variables, context), function () {
                return _await(mutateOptions.onSettled == null ? void 0 : mutateOptions.onSettled(data, null, variables, context), function () {
                  // Dispatch after handlers as the mutation could still fail
                  if (isLatest()) {
                    dispatch({
                      type: 'success',
                      data: data
                    });
                  }

                  return data;
                });
              });
            });
          });
        });
      });
    }, function (error) {
      getConsole().error(error);
      return _await(latestOptions.onError == null ? void 0 : latestOptions.onError(error, variables, context), function () {
        return _await(mutateOptions.onError == null ? void 0 : mutateOptions.onError(error, variables, context), function () {
          return _await(latestOptions.onSettled == null ? void 0 : latestOptions.onSettled(undefined, error, variables, context), function () {
            return _await(mutateOptions.onSettled == null ? void 0 : mutateOptions.onSettled(undefined, error, variables, context), function () {
              if (isLatest()) {
                dispatch({
                  type: 'error',
                  error: error
                });
              }

              if (mutateOptions.throwOnError || latestOptions.throwOnError) {
                throw error;
              }
            });
          });
        });
      });
    });
  }), [dispatch]);
  React.useEffect(function () {
    var latestOptions = latestOptionsRef.current;

    if (state.error && (latestOptions.useErrorBoundary || latestOptions.suspense)) {
      throw state.error;
    }
  }, [state.error]);
  var reset = React.useCallback(function () {
    dispatch({
      type: 'reset'
    });
  }, [dispatch]);
  return React.useMemo(function () {
    return [mutate, _extends({}, state, {
      reset: reset
    })];
  }, [mutate, state, reset]);
}