"use strict";

exports.__esModule = true;
exports.setBatchedUpdates = setBatchedUpdates;
exports.getBatchedUpdates = getBatchedUpdates;
exports.notifyManager = exports.NotifyManager = void 0;

var _utils = require("./utils");

// CLASS
var NotifyManager = /*#__PURE__*/function () {
  function NotifyManager() {
    this.queue = [];
    this.transactions = 0;
  }

  var _proto = NotifyManager.prototype;

  _proto.batch = function batch(callback) {
    this.transactions++;
    var result = callback();
    this.transactions--;

    if (!this.transactions) {
      this.flush();
    }

    return result;
  };

  _proto.schedule = function schedule(notify) {
    if (this.transactions) {
      this.queue.push(notify);
    } else {
      (0, _utils.scheduleMicrotask)(function () {
        notify();
      });
    }
  };

  _proto.flush = function flush() {
    var queue = this.queue;
    this.queue = [];

    if (queue.length) {
      (0, _utils.scheduleMicrotask)(function () {
        var batchedUpdates = getBatchedUpdates();
        batchedUpdates(function () {
          queue.forEach(function (notify) {
            notify();
          });
        });
      });
    }
  };

  return NotifyManager;
}(); // GETTERS AND SETTERS
// Default to a dummy "batch" implementation that just runs the callback


exports.NotifyManager = NotifyManager;

var batchedUpdates = function batchedUpdates(callback) {
  callback();
}; // Allow injecting another batching function later


function setBatchedUpdates(fn) {
  batchedUpdates = fn;
} // Supply a getter just to skip dealing with ESM bindings


function getBatchedUpdates() {
  return batchedUpdates;
} // SINGLETON


var notifyManager = new NotifyManager();
exports.notifyManager = notifyManager;