"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.queryReducer = queryReducer;
exports.Query = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _utils = require("./utils");

var _notifyManager = require("./notifyManager");

var _setConsole = require("./setConsole");

// CLASS
function _empty() {}

function _awaitIgnored(value, direct) {
  if (!direct) {
    return value && value.then ? value.then(_empty) : Promise.resolve();
  }
}

function _invoke(body, then) {
  var result = body();

  if (result && result.then) {
    return result.then(then);
  }

  return then(result);
}

function _await(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

function _catch(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
}

function _async(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

var Query = /*#__PURE__*/function () {
  function Query(config) {
    var _config$queryHash;

    this.setOptions(config.options);
    this.observers = [];
    this.cache = config.cache;
    this.queryKey = config.queryKey;
    this.queryHash = (_config$queryHash = config.queryHash) != null ? _config$queryHash : (0, _utils.hashQueryKey)(config.queryKey, this.options);
    this.state = getDefaultState(this.options);
    this.scheduleGc();
  }

  var _proto = Query.prototype;

  _proto.setOptions = function setOptions(options) {
    var _this$options$cacheTi;

    this.options = (0, _extends2.default)({}, this.defaultOptions, options); // Default to 5 minutes if not cache time is set

    this.cacheTime = Math.max(this.cacheTime || 0, (_this$options$cacheTi = this.options.cacheTime) != null ? _this$options$cacheTi : 5 * 60 * 1000);
  };

  _proto.setDefaultOptions = function setDefaultOptions(options) {
    this.defaultOptions = options;
  };

  _proto.dispatch = function dispatch(action) {
    var _this = this;

    this.state = queryReducer(this.state, action);

    _notifyManager.notifyManager.batch(function () {
      _this.observers.forEach(function (observer) {
        observer.onQueryUpdate(action);
      });

      _this.cache.notify(_this);
    });
  };

  _proto.scheduleGc = function scheduleGc() {
    var _this2 = this;

    this.clearGcTimeout();

    if (_utils.isServer || this.observers.length > 0 || !(0, _utils.isValidTimeout)(this.cacheTime)) {
      return;
    }

    this.gcTimeout = setTimeout(function () {
      _this2.remove();
    }, this.cacheTime);
  };

  _proto.cancel = function cancel(silent) {
    var promise = this.promise;

    if (promise && this.cancelFetch) {
      this.cancelFetch(silent);
      return promise.then(_utils.noop).catch(_utils.noop);
    }

    return Promise.resolve(undefined);
  };

  _proto.clearTimersObservers = function clearTimersObservers() {
    this.observers.forEach(function (observer) {
      observer.clearTimers();
    });
  };

  _proto.clearGcTimeout = function clearGcTimeout() {
    clearTimeout(this.gcTimeout);
    this.gcTimeout = undefined;
  };

  _proto.setData = function setData(updater, options) {
    var _this$options$isDataE, _this$options;

    var prevData = this.state.data; // Get the new data

    var data = (0, _utils.functionalUpdate)(updater, prevData); // Structurally share data between prev and new data if needed

    if (this.options.structuralSharing !== false) {
      data = (0, _utils.replaceEqualDeep)(prevData, data);
    } // Use prev data if an isDataEqual function is defined and returns `true`


    if ((_this$options$isDataE = (_this$options = this.options).isDataEqual) == null ? void 0 : _this$options$isDataE.call(_this$options, prevData, data)) {
      data = prevData;
    } // Try to determine if more data can be fetched


    var canFetchMore = hasMorePages(this.options, data); // Set data and mark it as cached

    this.dispatch({
      type: 'success',
      data: data,
      canFetchMore: canFetchMore,
      updatedAt: options == null ? void 0 : options.updatedAt
    });
    return data;
  };

  _proto.remove = function remove() {
    this.cache.remove(this);
  };

  _proto.destroy = function destroy() {
    this.clearGcTimeout();
    this.clearTimersObservers();
    this.cancel();
  };

  _proto.isActive = function isActive() {
    return this.observers.some(function (observer) {
      return observer.options.enabled !== false;
    });
  };

  _proto.isFetching = function isFetching() {
    return Boolean(this.promise);
  };

  _proto.isStale = function isStale() {
    return this.state.isInvalidated || !this.state.updatedAt || this.observers.some(function (observer) {
      return observer.getCurrentResult().isStale;
    });
  };

  _proto.isStaleByTime = function isStaleByTime(staleTime) {
    if (staleTime === void 0) {
      staleTime = 0;
    }

    return this.state.isInvalidated || !this.state.updatedAt || !(0, _utils.timeUntilStale)(this.state.updatedAt, staleTime);
  };

  _proto.onFocus = function onFocus() {
    this.onExternalEvent('focus');
  };

  _proto.onOnline = function onOnline() {
    this.onExternalEvent('online');
  };

  _proto.onExternalEvent = function onExternalEvent(type) {
    var _this$continueFetch;

    // Execute the first observer that wants to fetch on this event
    var fetchObserver = this.observers.find(function (observer) {
      var _observer$options = observer.options,
          enabled = _observer$options.enabled,
          refetchOnWindowFocus = _observer$options.refetchOnWindowFocus,
          refetchOnReconnect = _observer$options.refetchOnReconnect;

      var _observer$getCurrentR = observer.getCurrentResult(),
          isStale = _observer$getCurrentR.isStale;

      return enabled !== false && (type === 'focus' && (refetchOnWindowFocus === 'always' || refetchOnWindowFocus !== false && isStale) || type === 'online' && (refetchOnReconnect === 'always' || refetchOnReconnect !== false && isStale));
    });

    if (fetchObserver) {
      fetchObserver.refetch();
    } // Continue any paused fetch


    (_this$continueFetch = this.continueFetch) == null ? void 0 : _this$continueFetch.call(this);
  };

  _proto.subscribeObserver = function subscribeObserver(observer) {
    if (this.observers.indexOf(observer) !== -1) {
      return;
    }

    this.observers.push(observer); // Stop the query from being garbage collected

    this.clearGcTimeout();
    this.cache.notify(this);
  };

  _proto.unsubscribeObserver = function unsubscribeObserver(observer) {
    this.observers = this.observers.filter(function (x) {
      return x !== observer;
    });

    if (!this.observers.length) {
      // If the transport layer does not support cancellation
      // we'll let the query continue so the result can be cached
      if (this.isTransportCancelable) {
        this.cancel();
      }

      this.scheduleGc();
    }

    this.cache.notify(this);
  };

  _proto.invalidate = function invalidate() {
    if (!this.state.isInvalidated) {
      this.dispatch({
        type: 'invalidate'
      });
    }
  };

  _proto.fetch = function fetch(options, fetchOptions) {
    try {
      var _exit2 = false;

      var _this4 = this;

      return _invoke(function () {
        if (_this4.promise) {
          return function () {
            if ((fetchOptions == null ? void 0 : fetchOptions.fetchMore) && _this4.state.data) {
              // Silently cancel current fetch if the user wants to fetch more
              return _awaitIgnored(_this4.cancel(true));
            } else {
              // Return current promise if we are already fetching
              _exit2 = true;
              return _this4.promise;
            }
          }();
        }
      }, function (_result2) {
        if (_exit2) return _result2;

        // Update config if passed, otherwise the config from the last execution is used
        if (options) {
          _this4.setOptions(options);
        } // Use the options from the first observer with a query function if no function is found.
        // This can happen when the query is hydrated or created with setQueryData.


        if (!_this4.options.queryFn) {
          var observer = _this4.observers.find(function (x) {
            return x.options.queryFn;
          });

          if (observer) {
            _this4.setOptions(observer.options);
          }
        }

        options = _this4.options; // Get the query function params

        var params = (0, _utils.ensureArray)(_this4.queryKey);
        var filter = options.queryFnParamsFilter;
        params = filter ? filter(params) : params;
        _this4.promise = _async(function () {
          return _catch(function () {
            var data;
            return _invoke(function () {
              if (options.infinite) {
                return _await(_this4.startInfiniteFetch(options, params, fetchOptions), function (_this4$startInfiniteF) {
                  data = _this4$startInfiniteF;
                });
              } else {
                return _await(_this4.startFetch(options, params), function (_this4$startFetch) {
                  data = _this4$startFetch;
                });
              }
            }, function () {
              // Set success state
              _this4.setData(data); // Cleanup


              delete _this4.promise; // Return data

              return data;
            });
          }, function (error) {
            // Set error state if needed
            if (!((0, _utils.isCancelledError)(error) && error.silent)) {
              _this4.dispatch({
                type: 'error',
                error: error
              });
            } // Log error


            if (!(0, _utils.isCancelledError)(error)) {
              (0, _setConsole.getConsole)().error(error);
            } // Cleanup


            delete _this4.promise; // Propagate error

            throw error;
          });
        })();
        return _this4.promise;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.startFetch = function startFetch(options, params) {
    // Create function to fetch the data
    var queryFn = options.queryFn || defaultQueryFn;

    var fetchData = function fetchData() {
      return queryFn.apply(void 0, params);
    }; // Set to fetching state if not already in it


    if (!this.state.isFetching) {
      this.dispatch({
        type: 'fetch'
      });
    } // Try to fetch the data


    return this.tryFetchData(options, fetchData);
  };

  _proto.startInfiniteFetch = function startInfiniteFetch(options, params, fetchOptions) {
    var fetchMore = fetchOptions == null ? void 0 : fetchOptions.fetchMore;

    var _ref = fetchMore || {},
        previous = _ref.previous,
        fetchMoreVariable = _ref.fetchMoreVariable;

    var isFetchingMore = fetchMore ? previous ? 'previous' : 'next' : false;
    var prevPages = this.state.data || []; // Create function to fetch a page

    var fetchPage = _async(function (pages, prepend, cursor) {
      var lastPage = getLastPage(pages, prepend);

      if (typeof cursor === 'undefined' && typeof lastPage !== 'undefined' && options.getFetchMore) {
        cursor = options.getFetchMore(lastPage, pages);
      }

      var queryFn = options.queryFn || defaultQueryFn;
      return _await(queryFn.apply(void 0, params.concat([cursor])), function (page) {
        return prepend ? [page].concat(pages) : [].concat(pages, [page]);
      });
    }); // Create function to fetch the data


    var fetchData = function fetchData() {
      if (isFetchingMore) {
        return fetchPage(prevPages, previous, fetchMoreVariable);
      } else if (!prevPages.length) {
        return fetchPage([]);
      } else {
        var promise = fetchPage([]);

        for (var i = 1; i < prevPages.length; i++) {
          promise = promise.then(fetchPage);
        }

        return promise;
      }
    }; // Set to fetching state if not already in it


    if (!this.state.isFetching || this.state.isFetchingMore !== isFetchingMore) {
      this.dispatch({
        type: 'fetch',
        isFetchingMore: isFetchingMore
      });
    } // Try to get the data


    return this.tryFetchData(options, fetchData);
  };

  _proto.tryFetchData = function tryFetchData(options, fn) {
    var _this5 = this;

    return new Promise(function (outerResolve, outerReject) {
      var resolved = false;
      var continueLoop;
      var cancelTransport;

      var done = function done() {
        resolved = true;
        delete _this5.cancelFetch;
        delete _this5.continueFetch;
        delete _this5.isTransportCancelable; // End loop if currently paused

        continueLoop == null ? void 0 : continueLoop();
      };

      var resolve = function resolve(value) {
        done();
        outerResolve(value);
      };

      var reject = function reject(value) {
        done();
        outerReject(value);
      }; // Create callback to cancel this fetch


      _this5.cancelFetch = function (silent) {
        reject(new _utils.CancelledError(silent));
        cancelTransport == null ? void 0 : cancelTransport();
      }; // Create callback to continue this fetch


      _this5.continueFetch = function () {
        continueLoop == null ? void 0 : continueLoop();
      }; // Create loop function


      var run = _async(function () {
        return _catch(function () {
          // Execute query
          var promiseOrValue = fn(); // Check if the transport layer support cancellation

          if ((0, _utils.isCancelable)(promiseOrValue)) {
            cancelTransport = function cancelTransport() {
              try {
                promiseOrValue.cancel();
              } catch (_unused) {}
            };

            _this5.isTransportCancelable = true;
          } // Await data


          return _await(promiseOrValue, function (data) {
            // Resolve with data
            resolve(data);
          });
        }, function (error) {
          var _options$retry, _options$retryDelay;

          // Stop if the fetch is already resolved
          if (resolved) {
            return;
          } // Do we need to retry the request?


          var failureCount = _this5.state.failureCount;
          var retry = (_options$retry = options.retry) != null ? _options$retry : 3;
          var retryDelay = (_options$retryDelay = options.retryDelay) != null ? _options$retryDelay : _utils.defaultRetryDelay;
          var shouldRetry = retry === true || failureCount < retry || typeof retry === 'function' && retry(failureCount, error);

          if (!shouldRetry) {
            // We are done if the query does not need to be retried
            reject(error);
            return;
          } // Increase the failureCount


          _this5.dispatch({
            type: 'failed'
          }); // Delay


          return _await((0, _utils.sleep)((0, _utils.functionalUpdate)(retryDelay, failureCount) || 0), function () {
            // Pause retry if the document is not visible or when the device is offline
            return _invoke(function () {
              if (!(0, _utils.isDocumentVisible)() || !(0, _utils.isOnline)()) {
                return _awaitIgnored(new Promise(function (continueResolve) {
                  continueLoop = continueResolve;
                }));
              }
            }, function () {
              if (!resolved) {
                run();
              }
            }); // Try again if not resolved yet
          });
        });
      }); // Start loop


      run();
    });
  };

  return Query;
}();

exports.Query = Query;

function defaultQueryFn() {
  return Promise.reject();
}

function getLastPage(pages, previous) {
  return previous ? pages[0] : pages[pages.length - 1];
}

function hasMorePages(options, pages, previous) {
  if (options.infinite && options.getFetchMore && Array.isArray(pages)) {
    return Boolean(options.getFetchMore(getLastPage(pages, previous), pages));
  }
}

function getDefaultState(options) {
  var data = typeof options.initialData === 'function' ? options.initialData() : options.initialData;
  var hasData = typeof data !== 'undefined';
  return {
    canFetchMore: hasMorePages(options, data),
    data: data,
    dataUpdateCount: 0,
    error: null,
    errorUpdateCount: 0,
    failureCount: 0,
    isFetching: false,
    isFetchingMore: false,
    isInvalidated: false,
    status: hasData ? 'success' : 'idle',
    updatedAt: hasData ? Date.now() : 0
  };
}

function queryReducer(state, action) {
  var _action$updatedAt;

  switch (action.type) {
    case 'failed':
      return (0, _extends2.default)({}, state, {
        failureCount: state.failureCount + 1
      });

    case 'fetch':
      return (0, _extends2.default)({}, state, {
        failureCount: 0,
        isFetching: true,
        isFetchingMore: action.isFetchingMore || false,
        status: typeof state.data === 'undefined' ? 'loading' : 'success'
      });

    case 'success':
      return (0, _extends2.default)({}, state, {
        canFetchMore: action.canFetchMore,
        data: action.data,
        dataUpdateCount: state.dataUpdateCount + 1,
        error: null,
        failureCount: 0,
        isFetching: false,
        isFetchingMore: false,
        isInvalidated: false,
        status: 'success',
        updatedAt: (_action$updatedAt = action.updatedAt) != null ? _action$updatedAt : Date.now()
      });

    case 'error':
      return (0, _extends2.default)({}, state, {
        error: action.error,
        errorUpdateCount: state.errorUpdateCount + 1,
        failureCount: state.failureCount + 1,
        isFetching: false,
        isFetchingMore: false,
        status: 'error'
      });

    case 'invalidate':
      return (0, _extends2.default)({}, state, {
        isInvalidated: true
      });

    default:
      return state;
  }
}