"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.useMutation = useMutation;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _utils = require("./utils");

var _utils2 = require("../core/utils");

var _setConsole = require("../core/setConsole");

var _QueryClientProvider = require("./QueryClientProvider");

// HOOK
function _await(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

function _catch(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
}

function _async(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

function getDefaultState() {
  return (0, _extends2.default)({}, (0, _utils2.getStatusProps)('idle'), {
    data: undefined,
    error: null
  });
}

function reducer(state, action) {
  switch (action.type) {
    case 'reset':
      return getDefaultState();

    case 'loading':
      return (0, _extends2.default)({}, (0, _utils2.getStatusProps)('loading'), {
        data: undefined,
        error: null
      });

    case 'success':
      return (0, _extends2.default)({}, (0, _utils2.getStatusProps)('success'), {
        data: action.data,
        error: null
      });

    case 'error':
      return (0, _extends2.default)({}, (0, _utils2.getStatusProps)('error'), {
        data: undefined,
        error: action.error
      });

    default:
      return state;
  }
}

function useMutation(mutationFn, options) {
  if (options === void 0) {
    options = {};
  }

  var _React$useReducer = _react.default.useReducer(reducer, null, getDefaultState),
      state = _React$useReducer[0],
      unsafeDispatch = _React$useReducer[1];

  var dispatch = (0, _utils.useMountedCallback)(unsafeDispatch);
  var client = (0, _QueryClientProvider.useQueryClient)();
  var defaultedOptions = client.defaultMutationOptions(options);

  var latestMutationRef = _react.default.useRef(0);

  var latestMutationFnRef = _react.default.useRef(mutationFn);

  latestMutationFnRef.current = mutationFn;

  var latestOptionsRef = _react.default.useRef(defaultedOptions);

  latestOptionsRef.current = defaultedOptions;

  var mutate = _react.default.useCallback(_async(function (variables, mutateOptions) {
    if (mutateOptions === void 0) {
      mutateOptions = {};
    }

    dispatch({
      type: 'loading'
    });
    var mutationId = ++latestMutationRef.current;
    var latestOptions = latestOptionsRef.current;
    var latestMutationFn = latestMutationFnRef.current;

    var isLatest = function isLatest() {
      return latestMutationRef.current === mutationId;
    };

    var context;
    return _catch(function () {
      return _await(latestOptions.onMutate == null ? void 0 : latestOptions.onMutate(variables), function (_latestOptions$onMuta) {
        context = _latestOptions$onMuta;
        return _await(latestMutationFn(variables), function (data) {
          return _await(latestOptions.onSuccess == null ? void 0 : latestOptions.onSuccess(data, variables, context), function () {
            return _await(mutateOptions.onSuccess == null ? void 0 : mutateOptions.onSuccess(data, variables, context), function () {
              return _await(latestOptions.onSettled == null ? void 0 : latestOptions.onSettled(data, null, variables, context), function () {
                return _await(mutateOptions.onSettled == null ? void 0 : mutateOptions.onSettled(data, null, variables, context), function () {
                  // Dispatch after handlers as the mutation could still fail
                  if (isLatest()) {
                    dispatch({
                      type: 'success',
                      data: data
                    });
                  }

                  return data;
                });
              });
            });
          });
        });
      });
    }, function (error) {
      (0, _setConsole.getConsole)().error(error);
      return _await(latestOptions.onError == null ? void 0 : latestOptions.onError(error, variables, context), function () {
        return _await(mutateOptions.onError == null ? void 0 : mutateOptions.onError(error, variables, context), function () {
          return _await(latestOptions.onSettled == null ? void 0 : latestOptions.onSettled(undefined, error, variables, context), function () {
            return _await(mutateOptions.onSettled == null ? void 0 : mutateOptions.onSettled(undefined, error, variables, context), function () {
              if (isLatest()) {
                dispatch({
                  type: 'error',
                  error: error
                });
              }

              if (mutateOptions.throwOnError || latestOptions.throwOnError) {
                throw error;
              }
            });
          });
        });
      });
    });
  }), [dispatch]);

  _react.default.useEffect(function () {
    var latestOptions = latestOptionsRef.current;

    if (state.error && (latestOptions.useErrorBoundary || latestOptions.suspense)) {
      throw state.error;
    }
  }, [state.error]);

  var reset = _react.default.useCallback(function () {
    dispatch({
      type: 'reset'
    });
  }, [dispatch]);

  return _react.default.useMemo(function () {
    return [mutate, (0, _extends2.default)({}, state, {
      reset: reset
    })];
  }, [mutate, state, reset]);
}