import { Updater } from './utils';
import { IsFetchingMoreValue, QueryKey, QueryOptions, QueryStatus } from './types';
import { QueryCache } from './queryCache';
import { QueryObserver } from './queryObserver';
export interface QueryConfig<TData, TError, TQueryFnData> {
    cache: QueryCache;
    queryKey: QueryKey;
    queryHash?: string;
    options?: QueryOptions<TData, TError, TQueryFnData>;
}
export interface QueryState<TData, TError> {
    canFetchMore?: boolean;
    data?: TData;
    dataUpdateCount: number;
    error: TError | null;
    errorUpdateCount: number;
    failureCount: number;
    isFetching: boolean;
    isFetchingMore: IsFetchingMoreValue;
    isInvalidated: boolean;
    status: QueryStatus;
    updatedAt: number;
}
export interface FetchOptions {
    fetchMore?: FetchMoreOptions;
    throwOnError?: boolean;
}
interface FetchMoreOptions {
    fetchMoreVariable?: unknown;
    previous?: boolean;
}
interface SetDataOptions {
    updatedAt?: number;
}
interface FailedAction {
    type: 'failed';
}
interface FetchAction {
    type: 'fetch';
    isFetchingMore?: IsFetchingMoreValue;
}
interface SuccessAction<TData> {
    type: 'success';
    data: TData | undefined;
    canFetchMore?: boolean;
    updatedAt?: number;
}
interface ErrorAction<TError> {
    type: 'error';
    error: TError;
}
interface InvalidateAction {
    type: 'invalidate';
}
export declare type Action<TData, TError> = ErrorAction<TError> | FailedAction | FetchAction | InvalidateAction | SuccessAction<TData>;
export declare class Query<TData = unknown, TError = unknown, TQueryFnData = TData> {
    queryKey: QueryKey;
    queryHash: string;
    options: QueryOptions<TData, TError, TQueryFnData>;
    defaultOptions?: QueryOptions<TData, TError, TQueryFnData>;
    observers: QueryObserver<any, any, any, any>[];
    state: QueryState<TData, TError>;
    cacheTime: number;
    private cache;
    private promise?;
    private gcTimeout?;
    private cancelFetch?;
    private continueFetch?;
    private isTransportCancelable?;
    constructor(config: QueryConfig<TData, TError, TQueryFnData>);
    private setOptions;
    setDefaultOptions(options: QueryOptions<TData, TError, TQueryFnData>): void;
    private dispatch;
    private scheduleGc;
    cancel(silent?: boolean): Promise<void>;
    private clearTimersObservers;
    private clearGcTimeout;
    setData(updater: Updater<TData | undefined, TData>, options?: SetDataOptions): TData;
    remove(): void;
    destroy(): void;
    isActive(): boolean;
    isFetching(): boolean;
    isStale(): boolean;
    isStaleByTime(staleTime?: number): boolean;
    onFocus(): void;
    onOnline(): void;
    private onExternalEvent;
    subscribeObserver(observer: QueryObserver<any, any, any, any>): void;
    unsubscribeObserver(observer: QueryObserver<any, any, any, any>): void;
    invalidate(): void;
    fetch(options?: QueryOptions<TData, TError, TQueryFnData>, fetchOptions?: FetchOptions): Promise<TData>;
    private startFetch;
    private startInfiniteFetch;
    private tryFetchData;
}
export declare function queryReducer<TData, TError>(state: QueryState<TData, TError>, action: Action<TData, TError>): QueryState<TData, TError>;
export {};
