import { Query } from './query';
import { QueryOptions, QueryFunction, QueryKey, QueryStatus } from './types';
export interface QueryFilters {
    /**
     * Include or exclude active queries
     */
    active?: boolean;
    /**
     * Match query key exactly
     */
    exact?: boolean;
    /**
     * Include or exclude fresh queries
     */
    fresh?: boolean;
    /**
     * Include or exclude inactive queries
     */
    inactive?: boolean;
    /**
     * Include queries matching this predicate function
     */
    predicate?: (query: Query) => boolean;
    /**
     * Include queries matching this query key
     */
    queryKey?: QueryKey;
    /**
     * Include or exclude stale queries
     */
    stale?: boolean;
    /**
     * Include or exclude fetching queries
     */
    fetching?: boolean;
}
export declare type DataUpdateFunction<TInput, TOutput> = (input: TInput) => TOutput;
export declare type Updater<TInput, TOutput> = TOutput | DataUpdateFunction<TInput, TOutput>;
interface Cancelable {
    cancel(): void;
}
export declare class CancelledError {
    silent?: boolean;
    constructor(silent?: boolean);
}
export declare const isServer: boolean;
export declare function noop(): undefined;
export declare function functionalUpdate<TInput, TOutput>(updater: Updater<TInput, TOutput>, input: TInput): TOutput;
export declare function defaultRetryDelay(attempt: number): number;
export declare function defaultQueryKeySerializerFn(queryKey: QueryKey): string;
export declare function hashQueryKey(queryKey: QueryKey, options?: QueryOptions<any, any>): string;
export declare function deepIncludes(a: any, b: any): boolean;
export declare function isValidTimeout(value: any): value is number;
export declare function isDocumentVisible(): boolean;
export declare function isOnline(): boolean;
export declare function ensureArray<T>(value: T | T[]): T[];
export declare function uniq<T>(array: T[]): T[];
export declare function difference<T>(array1: T[], array2: T[]): T[];
export declare function replaceAt<T>(array: T[], index: number, value: T): T[];
export declare function timeUntilStale(updatedAt: number, staleTime?: number): number;
export declare function parseQueryArgs<TOptions extends QueryOptions<any, any>>(arg1: QueryKey | TOptions, arg2?: QueryFunction<any> | TOptions, arg3?: TOptions): TOptions;
export declare function parseFilterArgs<TFilters extends QueryFilters, TOptions = unknown>(arg1?: QueryKey | TFilters, arg2?: TFilters | TOptions, arg3?: TOptions): [TFilters, TOptions | undefined];
export declare function matchQuery(filters: QueryFilters, query: Query<any, any>): boolean;
/**
 * This function returns `a` if `b` is deeply equal.
 * If not, it will replace any deeply equal children of `b` with those of `a`.
 * This can be used for structural sharing between JSON values for example.
 */
export declare function replaceEqualDeep<T>(a: unknown, b: T): T;
/**
 * Shallow compare objects. Only works with objects that always have the same properties.
 */
export declare function shallowEqualObjects<T>(a: T, b: T): boolean;
export declare function isPlainObject(o: any): o is Object;
export declare function isQueryKey(value: any): value is QueryKey;
export declare function isCancelable(value: any): value is Cancelable;
export declare function isError(value: any): value is Error;
export declare function isCancelledError(value: any): value is CancelledError;
export declare function sleep(timeout: number): Promise<void>;
export declare function getStatusProps<T extends QueryStatus>(status: T): {
    status: T;
    isLoading: boolean;
    isSuccess: boolean;
    isError: boolean;
    isIdle: boolean;
};
export declare function createSetHandler(fn: () => void): (callback: (handler: () => void) => void) => void;
/**
 * Schedules a microtask.
 * This can be useful to schedule state updates after rendering.
 */
export declare function scheduleMicrotask(callback: () => void): void;
export {};
